@@ -13,10 +13,12 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #include <asm/sigcontext.h>
+#include "fault-linux-m68k.c"
 
 #define SIGSEGV_FAULT_HANDLER_ARGLIST  int sig, int code, struct sigcontext *scp
+#define SIGSEGV_FAULT_ADDRESS  (unsigned long) get_fault_addr (scp)
 #define SIGSEGV_FAULT_CONTEXT  scp
 #define SIGSEGV_FAULT_STACKPOINTER  scp->sc_usp
