@@ -1,5 +1,5 @@
 /* Fault handler information.
-   Copyright (C) 1993-1999, 2002  Bruno Haible <bruno@clisp.org>
+   Copyright (C) 1993-1999, 2002, 2008  Bruno Haible <bruno@clisp.org>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,9 +28,12 @@ sigsegv_deinstall_handler (void)
 {
 }
 
-void
-sigsegv_leave_handler (void)
+int
+sigsegv_leave_handler (void (*continuation) (void*, void*, void*),
+                       void* cont_arg1, void* cont_arg2, void* cont_arg3)
 {
+  (*continuation) (cont_arg1, cont_arg2, cont_arg3);
+  return 1;
 }
 
 int
