@@ -1,5 +1,5 @@
 /* Test the stack overflow handler.
-   Copyright (C) 2002-2005  Bruno Haible <bruno@clisp.org>
+   Copyright (C) 2002-2006, 2008  Bruno Haible <bruno@clisp.org>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,10 +13,15 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+
+#ifndef _MSC_VER
+# include <config.h>
+#endif
 
 #include "sigsegv.h"
 #include <stdio.h>
+#include <limits.h>
 
 #if HAVE_STACK_OVERFLOW_RECOVERY
 
@@ -25,9 +30,6 @@
   typedef int sigset_t;
 # define sigemptyset(set)
 # define sigprocmask(how,set,oldset)
-#else
-  /* Unix */
-# include "config.h"
 #endif
 
 #include <stddef.h> /* needed for NULL on SunOS4 */
@@ -39,35 +41,52 @@
 # include <sys/time.h>
 # include <sys/resource.h>
 #endif
+#ifndef SIGSTKSZ
+# define SIGSTKSZ 16384
+#endif
 
 jmp_buf mainloop;
 sigset_t mainsigset;
 
 volatile int pass = 0;
 
+static void
+stackoverflow_handler_continuation (void *arg1, void *arg2, void *arg3)
+{
+  int arg = (int) (long) arg1;
+  longjmp (mainloop, arg);
+}
+
 void
 stackoverflow_handler (int emergency, stackoverflow_context_t scp)
 {
   pass++;
   printf ("Stack overflow %d caught.\n", pass);
   sigprocmask (SIG_SETMASK, &mainsigset, NULL);
-  sigsegv_leave_handler ();
-  longjmp (mainloop, emergency ? -1 : pass);
+  sigsegv_leave_handler (stackoverflow_handler_continuation,
+                         (void *) (long) (emergency ? -1 : pass), NULL, NULL);
+}
+
+volatile int *
+recurse_1 (int n, volatile int *p)
+{
+  if (n < INT_MAX)
+    *recurse_1 (n + 1, p) += n;
+  return p;
 }
 
 int
-recurse (int n)
+recurse (volatile int n)
 {
-  if (n >= 0)
-    return n + recurse (n + 1);
-  else
-    return 0;
+  return *recurse_1 (n, &n);
 }
 
 int
 main ()
 {
-  char mystack[16384];
+  /* glibc says: Users should use SIGSTKSZ as the size of user-supplied
+     buffers.  */
+  char mystack[SIGSTKSZ];
   sigset_t emptyset;
 
 #if HAVE_SETRLIMIT && defined RLIMIT_STACK
